using Art;
using Gdk;
using GLib;
using Gnome;
using System;

/**
   @file CanvasWaypoint.cs
   @brief Can draw and manage a waypoint on a canvas
*/

/**
   @class CanvasWaypoint
   @brief Can draw and manage a waypoint on a canvas
*/

public class CanvasWaypoint {

	CanvasGroup group;
	CanvasPixbuf icon;
	CanvasText label;
	
	private Waypoint waypoint;
	public Waypoint Waypoint {
		set { 
			waypoint = value;
			/// @todo update locations of underlying items
			Console.WriteLine ("CanvasWaypoint:Waypoint -> Update requested");
		}
		get { return waypoint; }
	}

	/// The same icon will be used for all instances
	static Pixbuf waypoint_icon = new Pixbuf (null, "waypoint-icon.png");

#if false

	// Bounding box coordinates (in canvas coordinates)
	int x1, y1, x2, y2;

	public static new GLib.GType GType
		{
			get {
				if (gtype == GLib.GType.Invalid)
					gtype = RegisterGType (typeof (CanvasWaypoint));
				return gtype;
			}
		}

	protected override void Draw (Drawable drawable, int x, int y, int width, int height)
	{
		Console.WriteLine ("CanvasWaypoint:Draw the waypoint within xy {0},{1} and wh {2},{3}", x, y, width, height);
		Gdk.GC gc = new Gdk.GC (drawable);
		gc.Function = Function.Copy;
		gc.Foreground = new Gdk.Color (0xff, 0, 0);


		double lat = waypoint.Position.Latitude;
		double lon = waypoint.Position.Longitude;
		I2w (ref lat, ref lon);
		int xc, yc;
		Canvas.W2c (lat, lon, out xc, out yc);

		Console.WriteLine ("CanvasWaypoint:Draw()  Place pixbuf at {0},{1}", xc, yc);
		drawable.DrawPixbuf (gc, waypoint_icon, 0, 0, xc, yc, -1, -1, RgbDither.Normal, 0, 0);
		drawable.DrawRectangle (gc, false, xc, yc, 16, 16);

	}

	protected override double Point (double x, double y, int cx, int cy, out CanvasItem actual_item)
	{
		actual_item = this;

		/// @todo  lie that we are not here instead of figuring it out.
		return 1;
	}

	protected override void Realize () 
	{
		base.Realize ();
		Console.WriteLine ("CanvasWaypoint:Realize()");
	}

	protected override void Render (ref CanvasBuf buf) 
	{
		Console.WriteLine ("CanvasWaypoint:Render the waypoint");

		/// @todo  not implemented
#if false
		if (Svp != null) {
			Canvas.RenderSvp (buf, Svp, 0xFF0000);
		}
#endif
	}


	protected override void Update (double [] affine, ref Art.SVP ClipPath, int flags)
	{
		Console.WriteLine ("CanvasWaypoint:Update()");

		base.Update (affine, ref ClipPath, flags);


		Console.WriteLine ("Update: {0},{1} to {2},{3}", x1, y1, x2, y2);

		// Convert the coordinates to the right ones.
#if false
		double lat = waypoint.Position.Latitude;
		double lon = waypoint.Position.Longitude;
#else
		double lat = (double) ((GLib.Object) this).Data["lat"];
		double lon = (double) ((GLib.Object) this).Data["lon"];
#endif
		Console.WriteLine ("Got the actual properties");

		I2w (ref lat, ref lon);
		Console.WriteLine ("I2w done");
		int xc, yc;
		Canvas.W2c (lat, lon, out xc, out yc);
		Console.WriteLine ("W2c done");

		// @todo Use pixmap dimentions
		x1 = xc;
		x2 = xc;
		y1 = yc + 16;
		y2 = yc + 16;
		Console.WriteLine ("Update: {0},{1} to {2},{3}", x1, y1, x2, y2);

		Canvas.UpdateBbox (this, x1, y1, x2, y2);


		Console.WriteLine ("CanvasWaypoint:Update() done.");
	}
#endif

	public CanvasWaypoint (CanvasGroup group, Waypoint waypoint)
	{
		this.group = group;
		this.waypoint = waypoint;

		icon = new CanvasPixbuf (group);
		icon.Pixbuf = waypoint_icon;
		icon.WidthInPixels = true;
		icon.HeightInPixels = true;
		icon.X = waypoint.Position.Latitude;
		icon.Y = waypoint.Position.Longitude;

		label = new CanvasText (group);
		label.Text = waypoint.Name;
		label.X = waypoint.Position.Latitude;
		label.Y = waypoint.Position.Longitude;

	}

}


