using Gdk;
using Gtk;
using GtkSharp;
using Glade;
using Gnome;
using System;
using System.Text;

using System.Collections;
using System.Runtime.InteropServices;
using System.Text.RegularExpressions;

/// \file MainWindow.cs
/// \brief Definition of the main window and driver of the UI.


public class MainWindow {
	public static MainWindow Toplevel = null;

	Db db;

	[Glade.Widget] Gtk.Window main_window;
	[Glade.Widget] Gtk.VBox toolbar_vbox;
	[Glade.Widget] Gtk.VBox map_vbox;
	[Glade.Widget] Gnome.Canvas map_canvas;
	[Glade.Widget] Gtk.Label status_label;
	
	MapView map_view;


	//
	// Constructor
	//
	public MainWindow (Db db)
	{
		this.db = db;

		Mono.Posix.Catalog.Init ("gtracks", GTracks.Defines.LOCALE_DIR);
		Glade.XML gui = Glade.XML.FromAssembly ("gtracks.glade", "main_window", "");
		gui.Autoconnect (this);

		Gtk.Toolbar toolbar = new Gtk.Toolbar ();
		toolbar_vbox.PackStart (toolbar);

		ToolButton zoomin_button = new ToolButton (Gtk.Stock.ZoomIn);
		zoomin_button.Clicked += HandleZoomInCommand;
		toolbar.Insert (zoomin_button, -1);

		ToolButton zoomout_button = new ToolButton (Gtk.Stock.ZoomOut);
		zoomout_button.Clicked += HandleZoomOutCommand;
		toolbar.Insert (zoomout_button, -1);

		ToolButton zoomnormal_button = new ToolButton (Gtk.Stock.Zoom100);
		zoomnormal_button.Clicked += HandleZoomDefaultCommand;
		toolbar.Insert (zoomnormal_button, -1);

		// Create the map view
		map_canvas.ButtonPressEvent += HandleMapViewButtonPressEvent;
		map_canvas.MotionNotifyEvent += HandleMapViewMotionNotifyEvent;
		/// @todo add events for the scroll buttons, they don't work right now.
		
		map_canvas.AddEvents ((int) EventMask.PointerMotionMask);
		map_view = new MapView (db, map_canvas);

		// Set the initial status text
		status_label.Text = Mono.Posix.Catalog.GetString ("Welcome to GTracks");

		main_window.ShowAll ();
		main_window.Destroyed += HandleQuitCommand;
	}

	//
	// Handlers for the menu items
	//

	void HandleAbout (object sender, EventArgs args)
	{
		string [] authors = new string [] {
			"Hans de Graaff",
		};

		// Translators should localize the following string
		// * which will give them credit in the About box.
		// * E.g. "Martin Willemoes Hansen"
		string translators = Mono.Posix.Catalog.GetString ("translator-credits");
		if (System.String.Compare (translators, "translator-credits") == 0) {
			translators = null;
		}
		
		new About ("GTracks", 
				   GTracks.Defines.VERSION, 
				   "Copyright 2005 Hans de Graaff\nBased on F-Spot code",
				   null, authors, null, translators, null).Show();
	}


	void HandleCreateWaypoint (object sender, EventArgs args) {
		WaypointCommands.Create command = new WaypointCommands.Create (db.Waypoints, main_window);
		command.Execute ();
	}


	void HandleImportCommand (object sender, EventArgs args) {
		// Ask for a filename
		FileSelection fs = new FileSelection ("Select a file to import");
		fs.Complete ("*.gpx");
		fs.SelectMultiple = false;
		fs.Run ();
		fs.Hide ();

		// Import the filename
		GPXFile file = new GPXFile (db, fs.Selections[0]);
		file.Import ();

		map_view.Update ();
	}

	void HandleMapViewButtonPressEvent (object sender, Gtk.ButtonPressEventArgs args)
	{
		System.Console.WriteLine ("Button pressed on map view, updating");
	}

	void HandleMapViewMotionNotifyEvent (object sender, Gtk.MotionNotifyEventArgs args)
	{
		map_view.HandleMotionNotifyEvent (sender, args, status_label);
	}


	


	void HandleNotImplemented (object sender, EventArgs args) {
		// A not implemented handle. For now just ignore.
		System.Console.WriteLine ("This action is not implemented!");
	}

	void HandleQuitCommand (object sender, EventArgs args)
	{
		// FIXME
		// Should use Application.Quit(), but for that to work
		// we need to terminate the threads first too.
		Environment.Exit (0);
	}

	void HandleZoomInCommand (object sender, EventArgs args)
	{
		map_view.HandleZoomInCommand ();
	}

	void HandleZoomOutCommand (object sender, EventArgs args)
	{
		map_view.HandleZoomOutCommand ();
	}

	void HandleZoomDefaultCommand (object sender, EventArgs args) 
	{
		map_view.HandleZoomDefaultCommand ();
	}



}

